AVRE = LibStub("AceAddon-3.0"):NewAddon("AVRE", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
AVRE.MINOR_VERSION = tonumber(("$Revision: 11 $"):match("%d+"))
local L = LibStub("AceLocale-3.0"):GetLocale("AVRE")
local zones = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")
local AVRE = AVRE
-----



--[[
      VARIABLES ----------------------------------------------------------------------------------------------------------------
  ]]

local POLLING_FREQUENCY = 1.0
local testZone
local possibleWipe = false

AVRE.isEnabled = false
AVRE.inCombat = false
AVRE.ver = {}
AVRE.raid = {}
AVRE.enc = {}
AVRE.zones = {}
AVRE.scene = nil
AVRE.fight = nil

AVRE.modules = {
	options = {
		loaded = false,
	},
	encounters = {
		icecrown = {
			zone = zones["Icecrown Citadel"],
			addon = "AVRE_Icecrown",
			loaded = false,
		},
	},
}



--[[
      UPVALUES -----------------------------------------------------------------------------------------------------------------
  ]]
  
local _G = getfenv(0)
local pairs = _G.pairs
local strlower = _G.strlower
local strupper = _G.strupper
local format = _G.format



--[[
      UTILITIES ----------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:tcopy(to, from)
 for k, v in pairs(from) do
  if type(v) == "table" then
   to[k] = {}
   AVRE:tcopy(to[k], v)
  else
   to[k] = v
  end
 end
end


function AVRE:Colorize(s, color, highlight)
	if color and s then
		if highlight then
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 128 + 127, (color.g or 1) * 128 + 127, (color.b or 1) * 128 + 127, s)
		else
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 255, (color.g or 1) * 255, (color.b or 1) * 255, s)
		end
	else
		return s
	end
end



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnInitialize()
	-- Register database
 self.db = LibStub("AceDB-3.0"):New("AVREDB", self.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	
	-- Register for slash commands
	self:RegisterChatCommand("avre", "ChatCommand")
end


function AVRE:OnEnable()
	-- Register for events
	self:UnregisterAllEvents()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEvent")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnEvent")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "OnEvent")
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "OnEvent")
	self:RegisterEvent("CHAT_MSG_RAID_BOSS_EMOTE", "OnEvent")
	self:RegisterEvent("CHAT_MSG_MONSTER_SAY", "OnEvent")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnCombatEvent")

	-- Initialize
	self:LoadZone()
	self:UpdateRaid()
	
	-- Turn us on
	AVRE.isEnabled = self.db.profile.enabled

	--[===[@debug@
	self:Print(L["Loaded"])
	--@end-debug@]===]
end


function AVRE:OnDisable()
	-- Cancel all timers
	self:CancelAllTimers()

	-- Turn us off
	AVRE.isEnabled = false
	
	--[===[@debug@
	self:Print(L["Disabled"])
	--@end-debug@]===]
end


function AVRE:ChatCommand(input)
	if self.modules.options.loaded then InterfaceOptionsFrame_OpenToCategory(self.optionsFrame) end
end


function AVRE:OnProfileChanged(event, database, newProfileKey)
	--[===[@debug@
		self:Print(L["Profile modified, rebooting"])
	--@end-debug@]===]
 self:Disable()
 self:Enable() 
end



--[[
      EVENTS -------------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnEvent(event, ...)

	if event == "RAID_ROSTER_UPDATE" then
		self:UpdateRaid()
		
	elseif event == "ZONE_CHANGED_NEW_AREA" then
		self:LoadZone()
						
	elseif event == "PLAYER_REGEN_DISABLED" then
		if UnitInRaid("player") then
			self.inCombat = true
			self:ScheduleRepeatingTimer("OnTimer", POLLING_FREQUENCY)
			self:CheckForPull()
		end
			
	elseif event == "PLAYER_REGEN_ENABLED" then
		self.inCombat = false
		self:UpdateRaid()
		
	elseif event == "CHAT_MSG_MONSTER_YELL" then
		local msg = select(1, ...)
		self:OnMonsterMessage("yell", msg)
	
	elseif event == "CHAT_MSG_MONSTER_EMOTE" then
		local msg = select(1, ...)
		self:OnMonsterMessage("emote", msg)
	
	elseif event == "CHAT_MSG_RAID_BOSS_EMOTE" then
		local msg = select(1, ...)
		self:OnMonsterMessage("emote", msg)
	
	elseif event == "CHAT_MSG_MONSTER_SAY" then
		local msg = select(1, ...)
		self:OnMonsterMessage("say", msg)
					
 end
end


function AVRE:OnCombatEvent(event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, ...)
	if AVRE.fight and AVRE.isEnabled then

		if AVRE.fight.combatEvents and (AVRE.fight.combatEvents.ALL or AVRE.fight.combatEvents[eventType]) then
			if AVRE.fight.onCombat then
				AVRE.fight.onCombat(AVRE.fight, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, ...)
			end
		end
		
		if eventType == "UNIT_DIED" or eventType == "UNIT_DESTROYED" then
			if bit.band(dstGuid:sub(1, 5), 0x00F) == 3 or bit.band(dstGuid:sub(1, 5), 0x00F) == 5 then
				self:OnMobKill(tonumber(dstGuid:sub(9, 12), 16))
			end
		end

	end
end


--[[
      FIGHT LOGIC --------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnTimer()
	self:UpdateRaid()
end


function AVRE:UpdateRaid()
	if UnitInRaid("player") then
	
		for k, v in pairs(AVRE.raid) do
			AVRE.raid[k].valid = nil
		end
	
		-- Gather data about the raid
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			if playerName then
				if not AVRE.raid[playerName] then AVRE.raid[playerName] = {} end
				AVRE.raid[playerName].rank = rank
				AVRE.raid[playerName].subgroup = subgroup
				AVRE.raid[playerName].level = level
				AVRE.raid[playerName].class = class
				AVRE.raid[playerName].zone = zone
				AVRE.raid[playerName].online = online
				AVRE.raid[playerName].isDead = isDead
				AVRE.raid[playerName].role = role
				AVRE.raid[playerName].isML = isML
				AVRE.raid[playerName].valid = true
			end -- valid player at that index
		end -- player loop
		
		-- Delete players no longer in the raid
		for k, v in pairs(AVRE.raid) do
			if not v.valid then
				AVRE.raid[k] = nil
			end
		end
		
		-- Check for a wipe
		if AVRE.isEnabled and AVRE.fight and self:CheckForWipe() and not possibleWipe then
			possibleWipe = true
			local time = 5
			if AVRE.fight.wipeTime then time = AVRE.fight.wipeTime end
			self:ScheduleTimer(function()
				if possibleWipe then
					if AVRE:CheckForWipe() then
						possibleWipe = false
						if AVRE.fight.onWipe then AVRE.fight.onWipe(AVRE.fight) end
						AVRE:EndFight()
					end
				end
			end, time)
		end
		
	else
		AVRE.raid = {}
	end -- we're in a raid
end


function AVRE:CancelPossibleWipe()
	possibleWipe = false
end


function AVRE:TestZone(zone)
	testZone = zone
	self:LoadZone()
end


function AVRE:LoadZone()
	for k, v in pairs(self.modules.encounters) do
		if (GetRealZoneText() == v.zone or (testZone and testZone == v.zone)) and v.addon and not IsAddOnLoaded(v.addon) then
			local inInstance, instanceType = IsInInstance()
			if inInstance and instanceType == "raid" then
				self:ScheduleTimer(function() LoadAddOn(v.addon) end, 3)
			end
		end
	end
end


function AVRE:StartFight(fight, startDelay)
	local enabled = false
	if fight and fight.enable then
		if fight.enable.expansion and self.db.profile.enc[fight.enable.expansion] and self.db.profile.enc[fight.enable.expansion].enabled then
			if fight.enable.instance and self.db.profile.enc[fight.enable.expansion][fight.enable.instance] and self.db.profile.enc[fight.enable.expansion][fight.enable.instance].enabled then
				if fight.enable.fight and self.db.profile.enc[fight.enable.expansion][fight.enable.instance][fight.enable.fight] and self.db.profile.enc[fight.enable.expansion][fight.enable.instance][fight.enable.fight].enabled then
					enabled = true
				end
			end
		end
	end
	
	if enabled and AVRE.isEnabled then
		AVRE.scene = AVR:GetTempScene("AVR_ENCOUNTERS")
		AVRE.scene:ClearScene()
		AVRE.scene.visible = true
		possibleWipe = false
		fight.inCombat = true
		fight.startDelay = startDelay
		fight.dead = {}
		AVRE.fight = fight
		--[===[@debug@
		self:Print(format("Initiating fight: %s", fight.name))
		--@end-debug@]===]
		if fight.onStart then fight.onStart(fight) end
	end
end


function AVRE:EndFight()
	self:CancelAllTimers()
	if AVRE.fight then
		if AVRE.scene then
			AVRE.scene:ClearScene()
			AVRE.scene.visible = false
		end
		AVRE.fight.inCombat = false
		--[===[@debug@
		self:Print(format("Ending fight: %s", AVRE.fight.name))
		--@end-debug@]===]
		if AVRE.fight.onEnd then AVRE.fight.onEnd(AVRE.fight) end
		AVRE.fight = nil
	end
end


function AVRE:GetUnitCreatureId(unit)
	local guid = UnitGUID(unit)
	return (guid and tonumber(guid:sub(9, 12), 16)) or 0
end


function AVRE:GetCIDFromGUID(guid)
	return (guid and tonumber(guid:sub(9, 12), 16)) or 0
end


function AVRE:GetBossTarget(cid)
	for i = 1, GetNumRaidMembers() do
		if self:GetUnitCreatureId("raid"..i.."target") == cid then
			return UnitName("raid"..i.."targettarget"), "raid"..i.."targettarget"
		elseif self:GetUnitCreatureId("focus") == cid then
			return UnitName("focustarget"), "focustarget"
		end
	end
end



--[[
      PULL DETECTION -----------------------------------------------------------------------------------------------------------
  ]]
  
-- Combat-based detection (some raid member is targeting the boss, and the boss is affecting combat)
do
	local targetList = {}
	local function buildTargetList()
		local uId = ((GetNumRaidMembers() == 0) and "party") or "raid"
		for i = 0, math.max(GetNumRaidMembers(), GetNumPartyMembers()) do
			local id = (i == 0 and "target") or uId..i.."target"
			local guid = UnitGUID(id)
			if guid and (bit.band(guid:sub(1, 5), 0x00F) == 3 or bit.band(guid:sub(1, 5), 0x00F) == 5) then
				local cId = tonumber(guid:sub(9, 12), 16)
				targetList[cId] = id
			end
		end
	end

	local function clearTargetList()
		table.wipe(targetList)
	end

	local function delayed_checkForPull(mob, fight)
		if not fight.inCombat then
			buildTargetList()
			if targetList[mob] and UnitAffectingCombat(targetList[mob]) then
				AVRE:StartFight(fight, 3)
			end
			clearTargetList()
		end
	end
	
	local function checkForPull(mob, fight)
		local uId = targetList[mob]
		if uId and UnitAffectingCombat(uId) then
			AVRE:StartFight(fight, 0)
			return true
		elseif uId then
			AVRE:ScheduleTimer(function() delayed_checkForPull(mob, fight) end, 3)
		end
	end

	function AVRE:CheckForPull()
		if zones and (zones[GetRealZoneText()] or (testZone and zones[testZone])) then
			buildTargetList()
			for k, v in pairs(enc) do
				if v.pull and v.pull.type == "combat" and v.pull.npcs then
					for kk, vv in pairs(v.pull.npcs) do
						if checkForPull(vv, v) then break end
					end
				end
			clearTargetList()
			end
		end
	end
	
end


-- Yell-based detection (used for some vehicle fights, etc.)
function AVRE:OnMonsterMessage(type, msg)

	-- Pull detection
	if zones and (zones[GetRealZoneText()] or (testZone and zones[testZone])) then
		for k, v in pairs(enc) do
			if v.pull and v.pull.type == "yell" and v.pull.yells then
				for kk, vv in pairs(v.pull.yells) do
					if vv == msg then
						AVRE:StartFight(v, 0)
					end
				end
			end
		end
	end

	-- Defeat detection
	if zones and (zones[GetRealZoneText()] or (testZone and zones[testZone])) then
		for k, v in pairs(enc) do
			if v.defeat and v.defeat.type == "yell" and v.defeat.yells then
				for kk, vv in pairs(v.defeat.yells) do
					if vv == msg and v.inCombat then
						AVRE:EndFight()
					end
				end
			end
		end
	end

end
	


--[[
      WIPE DETECTION -----------------------------------------------------------------------------------------------------------
  ]]

function AVRE:CheckWipe()
	local wipe = true
	local uId = ((GetNumRaidMembers() == 0) and "party") or "raid"
	for i = 0, math.max(GetNumRaidMembers(), GetNumPartyMembers()) do
		local id = (i == 0 and "player") or uId..i
		if UnitAffectingCombat(id) and not UnitIsDeadOrGhost(id) then
			wipe = false
			break
		end
	end
	return wipe
end



--[[
      DEFEAT DETECTION ---------------------------------------------------------------------------------------------------------
  ]]

function AVRE:OnMobKill(id)
	if AVRE.fight and AVRE.fight.defeat == "death" then
		if not AVRE.fight.dead then AVRE.fight.dead = {} end
		if AVRE.fight.defeatNpcs then
			for k, v in pairs(AVRE.fight.defeatNpcs) do
				if id == v then
					AVRE.fight.dead[v] = true
				end
			end
			local allDead = true
			for k, v in pairs(AVRE.fight.defeatNpcs) do
				if not AVRE.fight.dead[v] then
					allDead = false
					break
				end
			end
			if allDead then
				if AVRE.fight.onDefeat then AVRE.fight.onDefeat(AVRE.fight) end
				self:EndFight()
			end
		end
	end
end



--[[
      API ----------------------------------------------------------------------------------------------------------------------
  ]]

function AVRE:RegisterEncounters(encounters)
	for k, v in pairs(encounters) do
		self:RegisterEncounter(k, v)
	end
end


function AVRE:RegisterEncounter(name, definition)
	AVRE.enc[name] = definition
	if definition.zone then zones[definition.zone] = true end
	AVRE.enc[name].inCombat = false
end
