local L = LibStub("AceLocale-3.0"):GetLocale("AVRE", false)
local AVRE = AVRE
if not AVRE or not AVR then return end
local mod = AVRE:NewModule("Icecrown", "AceConsole-3.0", "AceTimer-3.0")
local zone = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")["Icecrown Citadel"]


function mod:OnInitialize()
	AVRE.modules.encounters.icecrown.loaded = true
end


function mod:OnEnable()
	DEFAULT_CHAT_FRAME:AddMessage(format(L["AVR Encounters (%s) loaded"], zone))
	
	AVRE:RegisterEncounters({
	
		
-- FESTERGUT ----------------------------------------------------------------------------------------------------------

		festergut = {
			enable = {
				expansion = "wotlk",
				instance = "icecrown",
				fight = "festergut",
			},
			name = L["Festergut"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 36626 },
			},
			defeat = {
				type = "death",
				npcs = { 36626 },
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Gas Spore
				if spellId == 69279 and AVRE.db.profile.enc.wotlk.icecrown.festergut.gasSpore then
					AVR:AddCircleWarning(AVRE.scene, dstName, true, 8, 12, nil, nil, nil, nil, nil, true)
				end
			end,
		},


-- ROTFACE ------------------------------------------------------------------------------------------------------------

		rotface = {
			enable = {
				expansion = "wotlk",
				instance = "icecrown",
				fight = "rotface",
			},
			name = L["Rotface"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 36627 },
			},
			defeat = {
				type = "death",
				npcs = { 36627 },
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_CAST_START = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Unstable Ooze Explosion
				if spellId == 69839 and AVRE.db.profile.enc.wotlk.icecrown.rotface.explosion then
					AVR:AddCircleWarningEveryone(AVRE.scene, false, 6, 10, nil, nil, nil, 0.2, 10)
				end
			end,
		},


-- PUTRICIDE ----------------------------------------------------------------------------------------------------------

		putricide = {
			enable = {
				expansion = "wotlk",
				instance = "icecrown",
				fight = "putricide",
			},
			name = L["Professor Putricide"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 36678, 37562, 37697 },
			},
			defeat = {
				type = "death",
				npcs = { 36678 },
			},
			
			onStart = function(self)
				self.unboundMeshes = {}
				self.unboundMeshes2 = {}
				self.sicknessMeshes = {}
			end,
			
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_CAST_SUCCESS = true,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_REMOVED = true,
			},

			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				if eventType == "SPELL_CAST_SUCCESS" then

					-- Malleable Goo
					if spellId == 72615 or spellId == 72295 or spellId == 74280 or spellId == 74281 then
						if AVRE.db.profile.enc.wotlk.icecrown.putricide.malleable then
							AVRE:Print("Malleable Goo cast")
							self:ScheduleTimer(function()						
								local target = AVRE:GetBossTarget(36678)
								if not target then return end
								AVR:AddCircleWarning(AVRE.scene, target, false, 5, 5)
							end, 0.1)
						end
					end
					
				elseif eventType == "SPELL_AURA_APPLIED" then

					-- Unbound Plague applied
					if spellId == 72855 or spellId == 72856 then
						if AVRE.db.profile.enc.wotlk.icecrown.putricide.plague then
							local player = dstName
							local m = self.unboundMeshes[player]
							if m then m:Remove() end
							m = self.unboundMeshes2[player]
							if m then m:Remove() end
							
							_, _, _, _, _, duration, expirationTime = UnitDebuff(player, spellName)
							if not duration or not expirationTime then return end
							
							m = AVR:AddCircleWarning(AVRE.scene, player, true, 2, nil, 1, 0.5, 0, nil, nil, true)
							m:SetTimer(60, expirationTime)
							self.unboundMeshes[player] = m
							
							m = AVR:AddCircleWarning(AVRE.scene, player, true, 4, nil, 1, 0.5, 0, nil, nil, true)
							m:SetTimer(10)
							self.unboundMeshes2[player] = m
						end
						
					-- Plague Sickness applied
					elseif spellId == 70953 or spellId == 73117 then
						if AVRE.db.profile.enc.wotlk.icecrown.putricide.sick then
							local player = dstName
							local m = self.sicknessMeshes[player]
							if m then m:Remove() end
							m = AVR:AddCircleWarning(AVRE.scene, player, true, .6, nil, 0, 1, 0, nil, nil, false)
							self.sicknessMeshes[player] = m
						end
					end
					
				elseif eventType == "SPELL_AURA_REMOVED" then
				
					-- Unbound Plague removed
					if spellId == 72855 or spellId == 72856 then
						local player = dstName
						local m = self.unboundMeshes[player]
						if m then 
							m:Remove()
							self.unboundMeshes[player] = nil
						end
						local m = self.unboundMeshes2[player]
						if m then 
							m:Remove()
							self.unboundMeshes2[player] = nil
						end

					-- Plague Sickness removed
					elseif spellId == 70953 or spellId == 73117 then
						local player = dstName
						local m = self.sicknessMeshes[player]
						if m then 
							m:Remove()
							self.sicknessMeshes[player] = nil
						end
					end
					
				end
			end,
			
		},


-- SINDRAGOSA ---------------------------------------------------------------------------------------------------------

		sindragosa = {
			enable = {
				expansion = "wotlk",
				instance = "icecrown",
				fight = "sindragosa",
			},
			name = L["Sindragosa"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 36853, 37533, 37534 },
			},
			defeat = {
				type = "death",
				npcs = { 36853 },
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Frost Beacon
				if spellId == 70126 and AVRE.db.profile.enc.wotlk.icecrown.sindragosa.beacon then
					AVR:AddCircleWarning(AVRE.scene, dstName, true, 10, 7, nil, nil, nil, nil, nil, true)
				end
			end,
		},
		
		
	-- THE LICH KING ------------------------------------------------------------------------------------------------------

		lichking = {
			enable = {
				expansion = "wotlk",
				instance = "icecrown",
				fight = "lichking",
			},
			name = L["The Lich King"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 36597 },
			},
			defeat = {
				type = "death",
				npcs = { 36597 },
			},
			
			onStart = function(self)
				self.wipeTime = nil
			end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_CAST_START = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Defile
				if spellId == 72762 and AVRE.db.profile.enc.wotlk.icecrown.lichking.defile then
					self:ScheduleTimer(function()						
						local target = AVRE:GetBossTarget(36597)
						if not target then return end
						AVR:AddCircleWarning(AVRE.scene, target, true, 5, 2)
					end, 0.1)
				-- Shadow Trap
				elseif spellId == 73539 and AVRE.db.profile.enc.wotlk.icecrown.lichking.shadowtrap then
					self:ScheduleTimer(function()						
						local target = AVRE:GetBossTarget(36597)
						if not target then return end
						AVR:AddCircleWarning(AVRE.scene, target, false, 5, 2)
					end, 0.05)
				-- Fury of the Frostmourne; extend the wipe check timer
				elseif spellId == 72350 then
					self.wipeTime = 160
					AVRE:CancelPossibleWipe()
				end
			end,
		},


})
	
end
